import * as React from 'react';
import * as ReactDOM from 'react-dom';
import { NotieActions, NotieLevel, alert } from './actions';
import { NotieContainer, NotieMessage, NotieChoices, NotieButton, NotieOverlay } from './components';
const DEFAULT_STATE = {
    visible: false,
    locked: false,
    position: 'top',
    okBtnText: 'OK',
    cancelBtnText: 'Cancel'
};
let alreadyLoaded = false;
const DEFAULT_TIMEOUT = 5000;
export class Notie extends React.Component {
    constructor(props) {
        super(props);
        this.showNotie = (event) => {
            if (this.state.locked)
                return;
            const settings = event.detail;
            const toLock = settings.level === NotieLevel.CONFIRM || settings.level === NotieLevel.FORCE;
            if (settings.confirmReject && settings.confirmResolve) {
                this.confirmReject = settings.confirmReject;
                this.confirmResolve = settings.confirmResolve;
            }
            else {
                delete this.confirmReject;
                delete this.confirmResolve;
            }
            if (this.timeout) {
                clearTimeout(this.timeout);
                this.timeout = null;
            }
            if (this.state.visible) {
                this.hideNotie(() => alert(settings));
                return;
            }
            if (this.notieRoot)
                this.notieRoot.style.opacity = '1';
            this.setState(state => ({
                message: settings.message,
                level: settings.level,
                visible: true,
                locked: toLock,
                okBtnText: settings.yesBtnText || state.okBtnText,
                cancelBtnText: settings.noBtnText || state.cancelBtnText
            }));
            if (!toLock) {
                this.timeout = window.setTimeout(() => {
                    this.hideNotie();
                    this.timeout = null;
                }, settings.ttl || DEFAULT_TIMEOUT);
            }
        };
        this.hideNotie = (callback) => {
            this.setState(DEFAULT_STATE);
            this.transitionendCallback = () => {
                if (this.notieRoot)
                    this.notieRoot.style.opacity = '0';
                if (typeof callback === 'function')
                    callback();
                this.notieRoot &&
                    this.notieRoot.removeEventListener('transitionend', this
                        .transitionendCallback);
                this.transitionendCallback = null;
            };
            this.notieRoot &&
                this.notieRoot.addEventListener('transitionend', this.transitionendCallback);
        };
        this.handleYes = () => this.hideNotie(this.confirmResolve);
        this.handleNo = () => this.hideNotie(this.confirmReject);
        this.handleDismiss = () => {
            if (this.state.locked)
                return;
            if (this.timeout) {
                clearTimeout(this.timeout);
                this.timeout = null;
            }
            if (this.state.visible) {
                this.hideNotie();
            }
        };
        this.rootRef = (c) => (this.notieRoot = c);
        this.state = {
            message: '',
            level: NotieLevel.INFO,
            visible: false,
            locked: false,
            position: 'top',
            okBtnText: 'OK',
            cancelBtnText: 'Cancel'
        };
        this.root = document.createElement('div');
    }
    componentWillMount() {
        if (alreadyLoaded)
            throw new Error('<Notie /> is already loaded somwhere in your code. Please include it only once.');
        alreadyLoaded = true;
    }
    componentDidMount() {
        if (this.notieRoot)
            this.notieRoot.style.opacity = '0';
        window.addEventListener(NotieActions.SHOW, this.showNotie);
        document.body.appendChild(this.root);
    }
    shouldComponentUpdate(_, nextState) {
        return this.state.visible !== nextState.visible;
    }
    componentWillUnmount() {
        if (this.transitionendCallback) {
            this.notieRoot &&
                this.notieRoot.removeEventListener('transitionend', this.transitionendCallback);
            this.transitionendCallback = null;
        }
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = null;
        }
        if (alreadyLoaded)
            alreadyLoaded = false;
        window.removeEventListener(NotieActions.SHOW, this.showNotie);
        document.body.removeChild(this.root);
    }
    render() {
        const { message, level, visible, position } = this.state;
        const forced = level === NotieLevel.CONFIRM || level === NotieLevel.FORCE;
        return ReactDOM.createPortal(React.createElement("div", { ref: this.rootRef },
            forced && visible && React.createElement(NotieOverlay, null),
            React.createElement(NotieContainer, { active: visible, position: position, level: level, onClick: this.handleDismiss },
                React.createElement(NotieMessage, null, message),
                forced && (React.createElement(NotieChoices, null,
                    React.createElement(NotieButton, { level: level === NotieLevel.FORCE
                            ? NotieLevel.ERROR
                            : NotieLevel.SUCCESS, onClick: this.handleYes }, this.state.okBtnText),
                    level === NotieLevel.CONFIRM && (React.createElement(NotieButton, { level: NotieLevel.ERROR, onClick: this.handleNo }, this.state.cancelBtnText)))))), this.root);
    }
}
