"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOM = require("react-dom");
var actions_1 = require("./actions");
var components_1 = require("./components");
var DEFAULT_STATE = {
    visible: false,
    locked: false,
    position: 'top',
    okBtnText: 'OK',
    cancelBtnText: 'Cancel'
};
var alreadyLoaded = false;
var DEFAULT_TIMEOUT = 5000;
var Notie = /** @class */ (function (_super) {
    __extends(Notie, _super);
    function Notie(props) {
        var _this = _super.call(this, props) || this;
        _this.showNotie = function (event) {
            if (_this.state.locked)
                return;
            var settings = event.detail;
            var toLock = settings.level === actions_1.NotieLevel.CONFIRM || settings.level === actions_1.NotieLevel.FORCE;
            if (settings.confirmReject && settings.confirmResolve) {
                _this.confirmReject = settings.confirmReject;
                _this.confirmResolve = settings.confirmResolve;
            }
            else {
                delete _this.confirmReject;
                delete _this.confirmResolve;
            }
            if (_this.timeout) {
                clearTimeout(_this.timeout);
                _this.timeout = null;
            }
            if (_this.state.visible) {
                _this.hideNotie(function () { return actions_1.alert(settings); });
                return;
            }
            if (_this.notieRoot)
                _this.notieRoot.style.opacity = '1';
            _this.setState(function (state) { return ({
                message: settings.message,
                level: settings.level,
                visible: true,
                locked: toLock,
                okBtnText: settings.yesBtnText || state.okBtnText,
                cancelBtnText: settings.noBtnText || state.cancelBtnText
            }); });
            if (!toLock) {
                _this.timeout = window.setTimeout(function () {
                    _this.hideNotie();
                    _this.timeout = null;
                }, settings.ttl || DEFAULT_TIMEOUT);
            }
        };
        _this.hideNotie = function (callback) {
            _this.setState(DEFAULT_STATE);
            _this.transitionendCallback = function () {
                if (_this.notieRoot)
                    _this.notieRoot.style.opacity = '0';
                if (typeof callback === 'function')
                    callback();
                _this.notieRoot &&
                    _this.notieRoot.removeEventListener('transitionend', _this
                        .transitionendCallback);
                _this.transitionendCallback = null;
            };
            _this.notieRoot &&
                _this.notieRoot.addEventListener('transitionend', _this.transitionendCallback);
        };
        _this.handleYes = function () { return _this.hideNotie(_this.confirmResolve); };
        _this.handleNo = function () { return _this.hideNotie(_this.confirmReject); };
        _this.handleDismiss = function () {
            if (_this.state.locked)
                return;
            if (_this.timeout) {
                clearTimeout(_this.timeout);
                _this.timeout = null;
            }
            if (_this.state.visible) {
                _this.hideNotie();
            }
        };
        _this.rootRef = function (c) { return (_this.notieRoot = c); };
        _this.state = {
            message: '',
            level: actions_1.NotieLevel.INFO,
            visible: false,
            locked: false,
            position: 'top',
            okBtnText: 'OK',
            cancelBtnText: 'Cancel'
        };
        _this.root = document.createElement('div');
        return _this;
    }
    Notie.prototype.componentWillMount = function () {
        if (alreadyLoaded)
            throw new Error('<Notie /> is already loaded somwhere in your code. Please include it only once.');
        alreadyLoaded = true;
    };
    Notie.prototype.componentDidMount = function () {
        if (this.notieRoot)
            this.notieRoot.style.opacity = '0';
        window.addEventListener(actions_1.NotieActions.SHOW, this.showNotie);
        document.body.appendChild(this.root);
    };
    Notie.prototype.shouldComponentUpdate = function (_, nextState) {
        return this.state.visible !== nextState.visible;
    };
    Notie.prototype.componentWillUnmount = function () {
        if (this.transitionendCallback) {
            this.notieRoot &&
                this.notieRoot.removeEventListener('transitionend', this.transitionendCallback);
            this.transitionendCallback = null;
        }
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = null;
        }
        if (alreadyLoaded)
            alreadyLoaded = false;
        window.removeEventListener(actions_1.NotieActions.SHOW, this.showNotie);
        document.body.removeChild(this.root);
    };
    Notie.prototype.render = function () {
        var _a = this.state, message = _a.message, level = _a.level, visible = _a.visible, position = _a.position;
        var forced = level === actions_1.NotieLevel.CONFIRM || level === actions_1.NotieLevel.FORCE;
        return ReactDOM.createPortal(React.createElement("div", { ref: this.rootRef },
            forced && visible && React.createElement(components_1.NotieOverlay, null),
            React.createElement(components_1.NotieContainer, { active: visible, position: position, level: level, onClick: this.handleDismiss },
                React.createElement(components_1.NotieMessage, null, message),
                forced && (React.createElement(components_1.NotieChoices, null,
                    React.createElement(components_1.NotieButton, { level: level === actions_1.NotieLevel.FORCE
                            ? actions_1.NotieLevel.ERROR
                            : actions_1.NotieLevel.SUCCESS, onClick: this.handleYes }, this.state.okBtnText),
                    level === actions_1.NotieLevel.CONFIRM && (React.createElement(components_1.NotieButton, { level: actions_1.NotieLevel.ERROR, onClick: this.handleNo }, this.state.cancelBtnText)))))), this.root);
    };
    return Notie;
}(React.Component));
exports.Notie = Notie;
